@@ -35,6 +35,7 @@
 #include <stddef.h>
 #include <stdio.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <string.h>
 #include <ctype.h>
 #ifdef HAVE_SYS_TYPES_H
@@ -91,6 +92,12 @@
 #include "userinfo.h"
 #include "usermap.h"
 
+/* Socket file support for MacOS and FreeBSD */
+#if defined(__APPLE__) || defined(__FreeBSD__)
+#include <sys/socket.h>
+#include <sys/un.h>
+#endif
+
 /* Apple Structs */
 #ifdef __APPLE__
 #include <sys/param.h>
@@ -110,8 +117,8 @@ int flock(int fd, int operation);
 
 /* We pessimistically assume signed uid_t and gid_t in our overflow checks,
    mostly because supporting both cases would require a bunch more code. */
-static const uid_t UID_T_MAX = ((1LL << (sizeof(uid_t)*8-1)) - 1);
-static const gid_t GID_T_MAX = ((1LL << (sizeof(gid_t)*8-1)) - 1);
+static const int64_t UID_T_MAX = ((1LL << (sizeof(uid_t)*8-1)) - 1);
+static const int64_t GID_T_MAX = ((1LL << (sizeof(gid_t)*8-1)) - 1);
 static const int UID_GID_OVERFLOW_ERRNO = EIO;
 
 /* SETTINGS */
@@ -207,8 +214,8 @@ static struct Settings {
     bool direct_io;
 #endif
 
-    uid_t uid_offset;
-    gid_t gid_offset;
+    int64_t uid_offset;
+    int64_t gid_offset;
 
 } settings;
 
@@ -218,7 +225,7 @@ static bool bindfs_init_failed = false;
 
 /* PROTOTYPES */
 
-static int is_mirroring_enabled();
+static int is_mirroring_enabled(void);
 
 /* Checks whether the uid is to be the mirrored owner of all files. */
 static int is_mirrored_user(uid_t uid);
@@ -236,10 +243,11 @@ static int chown_new_file(const char *path, struct fuse_context *fc, int (*chown
 static int delete_file(const char *path, int (*target_delete_func)(const char *));
 
 /* Apply offsets with overflow checking. */
-static int apply_uid_offset(uid_t *uid);
-static int apply_gid_offset(gid_t *gid);
-static int unapply_uid_offset(uid_t *uid);
-static int unapply_gid_offset(gid_t *gid);
+static bool apply_uid_offset(uid_t *uid);
+static bool apply_gid_offset(gid_t *gid);
+static bool unapply_uid_offset(uid_t *uid);
+static bool unapply_gid_offset(gid_t *gid);
+static bool bounded_add(int64_t* a, int64_t b, int64_t max);
 
 #ifdef __linux__
 static size_t round_up_buffer_size_for_direct_io(size_t size);
@@ -249,7 +257,7 @@ static size_t round_up_buffer_size_for_direct_io(size_t size);
 #ifdef HAVE_FUSE_3
 static void *bindfs_init(struct fuse_conn_info *conn, struct fuse_config *cfg);
 #else
-static void *bindfs_init();
+static void *bindfs_init(struct fuse_conn_info *conn);
 #endif
 static void bindfs_destroy(void *private_data);
 #ifdef HAVE_FUSE_3
@@ -308,9 +316,11 @@ static int bindfs_read(const char *path, char *buf, size_t size, off_t offset,
                        struct fuse_file_info *fi);
 static int bindfs_write(const char *path, const char *buf, size_t size,
                         off_t offset, struct fuse_file_info *fi);
+#if defined(HAVE_FUSE_29) || defined(HAVE_FUSE_3)
 static int bindfs_lock(const char *path, struct fuse_file_info *fi, int cmd,
                        struct flock *lock);
 static int bindfs_flock(const char *path, struct fuse_file_info *fi, int op);
+#endif
 #ifdef HAVE_FUSE_3
 static int bindfs_ioctl(const char *path, int cmd, void *arg,
                         struct fuse_file_info *fi, unsigned int flags,
@@ -321,6 +331,9 @@ static int bindfs_ioctl(const char *path, int cmd, void *arg,
                         void *data);
 #endif
 static int bindfs_statfs(const char *path, struct statvfs *stbuf);
+#if __APPLE__
+static int bindfs_statfs_x(const char *path, struct statfs *stbuf);
+#endif
 static int bindfs_release(const char *path, struct fuse_file_info *fi);
 static int bindfs_fsync(const char *path, int isdatasync,
                         struct fuse_file_info *fi);
@@ -332,14 +345,14 @@ static int process_option(void *data, const char *arg, int key,
                           struct fuse_args *outargs);
 static int parse_mirrored_users(char* mirror);
 static int parse_user_map(UserMap *map, UserMap *reverse_map, char *spec);
-static char *get_working_dir();
-static void maybe_stdout_stderr_to_file();
+static char *get_working_dir(void);
+static void maybe_stdout_stderr_to_file(void);
 
 /* Sets up handling of SIGUSR1. */
-static void setup_signal_handling();
+static void setup_signal_handling(void);
 static void signal_handler(int sig);
 
-static void atexit_func();
+static void atexit_func(void);
 
 /* 
 Ignore some options (starting with -o) 
@@ -351,7 +364,7 @@ are special ones interpreted by systemd in /etc/fstab
 struct fuse_args filter_special_opts(struct fuse_args *args);
 static bool keep_option(const char* opt);
 
-static int is_mirroring_enabled()
+static int is_mirroring_enabled(void)
 {
     return settings.num_mirrored_users + settings.num_mirrored_members > 0;
 }
@@ -435,9 +448,9 @@ static int getattr_common(const char *procpath, struct stat *stbuf)
     }
 
     /* Report user-defined owner/group if specified */
-    if (settings.new_uid != -1)
+    if (settings.new_uid != (uid_t)-1)
         stbuf->st_uid = settings.new_uid;
-    if (settings.new_gid != -1)
+    if (settings.new_gid != (gid_t)-1)
         stbuf->st_gid = settings.new_gid;
 
     /* Mirrored user? */
@@ -538,12 +551,12 @@ static int chown_new_file(const char *path, struct fuse_context *fc, int (*chown
         }
     }
 
-    if (settings.create_for_uid != -1)
+    if (settings.create_for_uid != (uid_t)-1)
         file_owner = settings.create_for_uid;
-    if (settings.create_for_gid != -1)
+    if (settings.create_for_gid != (gid_t)-1)
         file_group = settings.create_for_gid;
 
-    if ((file_owner != -1) || (file_group != -1)) {
+    if ((file_owner != (uid_t)-1) || (file_group != (gid_t)-1)) {
         if (chown_func(path, file_owner, file_group) == -1) {
             DPRINTF("Failed to chown new file or directory (%d)", errno);
         }
@@ -625,40 +638,57 @@ static int delete_file(const char *path, int (*target_delete_func)(const char *)
     return 0;
 }
 
-static int apply_uid_offset(uid_t *uid) {
-    if (*uid > UID_T_MAX - settings.uid_offset) {
-        DPRINTF("UID %lld overflowed while applying offset", (long long)*uid);
-        return 0;
+static bool apply_uid_offset(uid_t *uid) {
+    int64_t value = (int64_t)*uid;
+    if (bounded_add(&value, settings.uid_offset, UID_T_MAX)) {
+        *uid = (uid_t)value;
+        return true;
+    } else {
+        DPRINTF("UID %ld out of bounds after applying offset", value);
+        return false;
     }
-    *uid += settings.uid_offset;
-    return 1;
 }
 
-static int apply_gid_offset(gid_t *gid) {
-    if (*gid > GID_T_MAX - settings.gid_offset) {
-        DPRINTF("GID %lld overflowed while applying offset", (long long)*gid);
-        return 0;
+static bool apply_gid_offset(gid_t *gid) {
+    int64_t value = (int64_t)*gid;
+    if (bounded_add(&value, settings.gid_offset, GID_T_MAX)) {
+        *gid = (uid_t)value;
+        return true;
+    } else {
+        DPRINTF("GID %ld out of bounds after applying offset", value);
+        return false;
     }
-    *gid += settings.gid_offset;
-    return 1;
 }
 
-static int unapply_uid_offset(uid_t *uid) {
-    if (*uid < settings.uid_offset) {
-        DPRINTF("UID %lld underflowed while unapplying offset", (long long)*uid);
-        return 0;
+static bool unapply_uid_offset(uid_t *uid) {
+    int64_t value = (int64_t)*uid;
+    if (bounded_add(&value, -settings.uid_offset, UID_T_MAX)) {
+        *uid = (uid_t)value;
+        return true;
+    } else {
+        DPRINTF("UID %ld out of bounds after unapplying offset", value);
+        return false;
     }
-    *uid -= settings.uid_offset;
-    return 1;
 }
 
-static int unapply_gid_offset(gid_t *gid) {
-  if (*gid < settings.gid_offset) {
-        DPRINTF("GID %lld underflowed while unapplying offset", (long long)*gid);
-        return 0;
+static bool unapply_gid_offset(gid_t *gid) {
+    int64_t value = (int64_t)*gid;
+    if (bounded_add(&value, -settings.gid_offset, GID_T_MAX)) {
+        *gid = (uid_t)value;
+        return true;
+    } else {
+        DPRINTF("GID %ld out of bounds after unapplying offset", value);
+        return false;
     }
-    *gid -= settings.gid_offset;
-    return 1;
+}
+
+static bool bounded_add(int64_t* a, int64_t b, int64_t max) {
+    int64_t result;
+    if (!__builtin_add_overflow(*a, b, &result) && 0 <= result && result <= max) {
+        *a = result;
+        return true;
+    }
+    return false;
 }
 
 #ifdef __linux__
@@ -679,11 +709,11 @@ static size_t round_up_buffer_size_for_direct_io(size_t size)
 #ifdef HAVE_FUSE_3
 static void *bindfs_init(struct fuse_conn_info *conn, struct fuse_config *cfg)
 #else
-static void *bindfs_init()
+static void *bindfs_init(struct fuse_conn_info *conn)
 #endif
 {
-    #ifdef HAVE_FUSE_3
     (void) conn;
+    #ifdef HAVE_FUSE_3
     cfg->use_ino = 1;
 
     // Disable caches so changes in base FS are visible immediately.
@@ -722,6 +752,7 @@ static void *bindfs_init()
 
 static void bindfs_destroy(void *private_data)
 {
+    (void)private_data;
 }
 
 #ifdef HAVE_FUSE_3
@@ -733,6 +764,9 @@ static int bindfs_getattr(const char *path, struct stat *stbuf)
 {
     int res;
     char *real_path;
+#ifdef HAVE_FUSE_3
+    (void)fi;
+#endif
 
     real_path = process_path(path, true);
     if (real_path == NULL)
@@ -799,6 +833,11 @@ static int bindfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
                           off_t offset, struct fuse_file_info *fi)
 #endif
 {
+    (void)offset;
+    (void)fi;
+#ifdef HAVE_FUSE_3
+    (void)flags;
+#endif
     char *real_path = process_path(path, true);
     if (real_path == NULL) {
         return -errno;
@@ -901,10 +940,36 @@ static int bindfs_mknod(const char *path, mode_t mode, dev_t rdev)
 
     mode = permchain_apply(settings.create_permchain, mode);
 
-    if (S_ISFIFO(mode))
+    if (S_ISFIFO(mode)) {
         res = mkfifo(real_path, mode);
-    else
+#if defined(__APPLE__) || defined(__FreeBSD__)
+    } else if (S_ISSOCK(mode)) {
+        struct sockaddr_un su;
+        int fd;
+
+        if (strlen(real_path) >= sizeof(su.sun_path)) {
+            errno = ENAMETOOLONG;
+            return -1;
+        }
+        fd = socket(AF_UNIX, SOCK_STREAM, 0);
+        if (fd >= 0) {
+            /*
+             * We must bind the socket to the underlying file
+             * system to create the socket file, even though
+             * we'll never listen on this socket.
+             */
+            su.sun_family = AF_UNIX;
+            strncpy(su.sun_path, real_path, sizeof(su.sun_path));
+            res = bind(fd, (struct sockaddr*)&su, sizeof(su));
+            close(fd);
+        } else {
+            res = -1;
+        }
+#endif
+    } else {
         res = mknod(real_path, mode, rdev);
+    }
+
     if (res == -1) {
         free(real_path);
         return -errno;
@@ -1062,6 +1127,9 @@ static int bindfs_chmod(const char *path, mode_t mode)
     struct stat st;
     mode_t diff = 0;
     char *real_path;
+#ifdef HAVE_FUSE_3
+    (void)fi;
+#endif
 
     real_path = process_path(path, true);
     if (real_path == NULL)
@@ -1127,8 +1195,11 @@ static int bindfs_chown(const char *path, uid_t uid, gid_t gid)
 {
     int res;
     char *real_path;
+#ifdef HAVE_FUSE_3
+    (void)fi;
+#endif
 
-    if (uid != -1) {
+    if (uid != (uid_t)-1) {
         switch (settings.chown_policy) {
         case CHOWN_NORMAL:
             uid = usermap_get_uid_or_default(settings.usermap_reverse, uid, uid);
@@ -1144,7 +1215,7 @@ static int bindfs_chown(const char *path, uid_t uid, gid_t gid)
         }
     }
 
-    if (gid != -1) {
+    if (gid != (gid_t)-1) {
         switch (settings.chgrp_policy) {
         case CHGRP_NORMAL:
             gid = usermap_get_gid_or_default(settings.usermap_reverse, gid, gid);
@@ -1160,7 +1231,7 @@ static int bindfs_chown(const char *path, uid_t uid, gid_t gid)
         }
     }
 
-    if (uid != -1 || gid != -1) {
+    if (uid != (uid_t)-1 || gid != (gid_t)-1) {
         real_path = process_path(path, true);
         if (real_path == NULL)
             return -errno;
@@ -1183,6 +1254,9 @@ static int bindfs_truncate(const char *path, off_t size)
 {
     int res;
     char *real_path;
+#ifdef HAVE_FUSE_3
+    (void)fi;
+#endif
 
     real_path = process_path(path, true);
     if (real_path == NULL)
@@ -1219,6 +1293,9 @@ static int bindfs_utimens(const char *path, const struct timespec ts[2])
 {
     int res;
     char *real_path;
+#ifdef HAVE_FUSE_3
+    (void)fi;
+#endif
 
     real_path = process_path(path, true);
     if (real_path == NULL)
@@ -1379,10 +1456,12 @@ static int bindfs_write(const char *path, const char *buf, size_t size,
     return res;
 }
 
+#if defined(HAVE_FUSE_29) || defined(HAVE_FUSE_3)
 /* This callback is only installed if lock forwarding is enabled. */
 static int bindfs_lock(const char *path, struct fuse_file_info *fi, int cmd,
                        struct flock *lock)
 {
+  (void)path;
   int res = fcntl(fi->fh, cmd, lock);
   if (res == -1) {
     return -errno;
@@ -1393,12 +1472,14 @@ static int bindfs_lock(const char *path, struct fuse_file_info *fi, int cmd,
 /* This callback is only installed if lock forwarding is enabled. */
 static int bindfs_flock(const char *path, struct fuse_file_info *fi, int op)
 {
+    (void)path;
     int res = flock(fi->fh, op);
     if (res == -1) {
         return -errno;
     }
     return 0;
 }
+#endif
 
 #ifdef HAVE_FUSE_3
 static int bindfs_ioctl(const char *path, int cmd, void *arg,
@@ -1410,6 +1491,9 @@ static int bindfs_ioctl(const char *path, int cmd, void *arg,
                         void *data)
 #endif
 {
+    (void)path;
+    (void)arg;
+    (void)flags;
     int res = ioctl(fi->fh, cmd, data);
     if (res == -1) {
       return -errno;
@@ -1434,6 +1518,25 @@ static int bindfs_statfs(const char *path, struct statvfs *stbuf)
     return 0;
 }
 
+#if __APPLE__
+static int bindfs_statfs_x(const char *path, struct statfs *stbuf)
+{
+    int res;
+    char *real_path;
+
+    real_path = process_path(path, true);
+    if (real_path == NULL)
+        return -errno;
+
+    res = statfs(real_path, stbuf);
+    free(real_path);
+    if (res == -1)
+        return -errno;
+
+    return 0;
+}
+#endif
+
 static int bindfs_release(const char *path, struct fuse_file_info *fi)
 {
     (void) path;
@@ -1575,7 +1678,7 @@ static int bindfs_listxattr(const char *path, char* list, size_t size)
                 }
                 curr += thislen;
                 len += thislen;
-            } while (len < res);
+            } while (len < (size_t)res);
         } else {
             // TODO: https://github.com/osxfuse/fuse/blob/master/example/fusexmp_fh.c
             // had this commented out bit here o_O
@@ -1672,6 +1775,9 @@ static struct fuse_operations bindfs_oper = {
     .ioctl      = bindfs_ioctl,
 #endif
     .statfs     = bindfs_statfs,
+#ifdef __APPLE__
+    .statfs_x   = bindfs_statfs_x,
+#endif
     .release    = bindfs_release,
     .fsync      = bindfs_fsync,
 #ifdef HAVE_SETXATTR
@@ -1815,6 +1921,8 @@ enum OptionKey {
 static int process_option(void *data, const char *arg, int key,
                           struct fuse_args *outargs)
 {
+    (void)data;
+    (void)outargs;
     switch ((enum OptionKey)key)
     {
     case OPTKEY_HELP:
@@ -2200,7 +2308,7 @@ fail:
     return 0;
 }
 
-static void maybe_stdout_stderr_to_file()
+static void maybe_stdout_stderr_to_file(void)
 {
     /* TODO: make this a command line option. */
 #if 0
@@ -2223,7 +2331,7 @@ static void maybe_stdout_stderr_to_file()
 #endif
 }
 
-static char *get_working_dir()
+static char *get_working_dir(void)
 {
     size_t buf_size = 4096;
     char* buf = malloc(buf_size);
@@ -2234,7 +2342,7 @@ static char *get_working_dir()
     return buf;
 }
 
-static void setup_signal_handling()
+static void setup_signal_handling(void)
 {
     struct sigaction sa;
     sa.sa_handler = signal_handler;
@@ -2246,10 +2354,11 @@ static void setup_signal_handling()
 
 static void signal_handler(int sig)
 {
+    (void)sig;
     invalidate_user_cache();
 }
 
-static void atexit_func()
+static void atexit_func(void)
 {
     free(settings.mntsrc);
     free(settings.mntdest);
@@ -2623,7 +2732,7 @@ int main(int argc, char *argv[])
             return 1;
         }
         char* endptr = od.uid_offset;
-        settings.uid_offset = strtoul(od.uid_offset, &endptr, 10);
+        settings.uid_offset = strtoll(od.uid_offset, &endptr, 10);
         if (*endptr != '\0') {
             fprintf(stderr, "Error: Value of --uid-offset must be an integer.\n");
             return 1;
@@ -2640,7 +2749,7 @@ int main(int argc, char *argv[])
             return 1;
         }
         char* endptr = od.gid_offset;
-        settings.gid_offset = strtoul(od.gid_offset, &endptr, 10);
+        settings.gid_offset = strtoll(od.gid_offset, &endptr, 10);
         if (*endptr != '\0') {
             fprintf(stderr, "Error: Value of --gid-offset must be an integer.\n");
             return 1;
