@@ -35,6 +35,6 @@ int user_uid(const char *username, uid_t *ret);
 int group_gid(const char *groupname, gid_t *ret);
 
 int user_belongs_to_group(uid_t uid, gid_t gid);
-void invalidate_user_cache(); /* safe to call from signal handler */
+void invalidate_user_cache(void); /* safe to call from signal handler */
 
 #endif
