@@ -25,13 +25,17 @@ $LOAD_PATH << (ENV['srcdir'] || '.')
 
 require 'common.rb'
 require 'etc'
+require 'socket'
 require 'tempfile'
 
 include Errno
 
 $have_fuse_3 = Proc.new do
   system("pkg-config --exists fuse3")
-  $?.success?
+  $?.success? || Proc.new do
+    system("pkg-config --exists fuse-t")
+    $?.success?
+  end.call
 end.call
 $have_fuse_3_readdir_bug = $have_fuse_3 && Proc.new do
   system("pkg-config --max-version=3.10.1 fuse3")
@@ -64,6 +68,9 @@ $nobody_gid = nobody_gid = Etc.getpwnam('nobody').gid
 $nobody_group = nobody_group = Etc.getgrgid(nobody_gid).name
 $root_group = root_group = Etc.getgrgid(0).name
 
+$user_1k = user_1k = begin Etc.getpwuid(1000).name; rescue; nil; end
+$user_1k_group = user_1k_group = begin Etc.getgrgid(Etc.getpwuid(1000).gid).name; rescue; nil; end
+
 $tests_dir = File.realpath('.')
 
 
@@ -467,8 +474,25 @@ root_testenv("--uid-offset=2 --gid-offset=20", :title => "file creation with --u
     assert { File.stat('mnt/file').gid == File.stat('src/file').gid + 20 }
 end
 
+# This test requires user 1k to actually exist so we can sudo to it
+if user_1k
+    root_testenv("--uid-offset=-2 --gid-offset=-20", :title => "file creation with negative --uid-offset and --gid-offset") do
+        chown(user_1k, user_1k_group, 'src')
+        chmod(0777, 'src')
+        `sudo -u #{user_1k} -g #{user_1k_group} touch mnt/file`
+
+        assert { File.stat('src/file').uid == 1002 }
+        assert { File.stat('mnt/file').uid == 1000 }
+        assert { File.stat('mnt/file').gid == File.stat('src/file').gid - 20 }
+    end
+end
+
 root_testenv("--uid-offset=2 --gid-offset=20", :title => "chown/chgrp with --uid-offset and --gid-offset") do
     touch('src/file')
+    # Avoid I/O error due to uid and gid = 0 not being mappable with negative offsets
+    chown(1000, 1000, 'src')
+    chown(1000, 1000, 'src/file')
+
     chown(6, 25, 'mnt/file')
 
     assert { File.stat('src/file').uid == 4 }
@@ -477,6 +501,20 @@ root_testenv("--uid-offset=2 --gid-offset=20", :title => "chown/chgrp with --uid
     assert { File.stat('mnt/file').gid == 25 }
 end
 
+root_testenv("--uid-offset=-2 --gid-offset=-20", :title => "chown/chgrp with negative --uid-offset and --gid-offset") do
+    touch('src/file')
+    # Avoid I/O error due to uid and gid = 0 not being mappable with negative offsets
+    chown(1000, 1000, 'src')
+    chown(1000, 1000, 'src/file')
+
+    chown(6, 25, 'mnt/file')
+
+    assert { File.stat('src/file').uid == 8 }
+    assert { File.stat('src/file').gid == 45 }
+    assert { File.stat('mnt/file').uid == 6 }
+    assert { File.stat('mnt/file').gid == 25 }
+end
+
 testenv("", :title => "preserves inode numbers") do
     touch('src/file')
     mkdir('src/dir')
@@ -922,7 +960,20 @@ testenv("-ouser -onofail,nouser,,,delete-deny -o users -o auto,rename-deny,noaut
   assert_exception(EPERM) { chmod(0777, 'mnt/file') }
 end
 
-# FIXME: this stuff around testenv is a hax, and testenv may also exit(), which defeats the 'ensure' below.
+# Issue #132 / PR #133
+testenv("", :title => "socket files") do
+  UNIXServer.open("mnt/sock") do |server|
+    UNIXSocket.open("mnt/sock") do |client|
+      socket = server.accept
+      socket.write("hello")
+      socket.close
+      result = client.read
+      assert { result == "hello" }
+    end
+  end
+end
+
+# FIXME: this stuff around testenv is a hack, and testenv may also exit(), which defeats the 'ensure' below.
 # the test setup ought to be refactored. It might well use MiniTest or something.
 # TODO: support FreeBSD in this test (different group management commands)
 if Process.uid == 0 && `uname`.strip == 'Linux'
